// Copyright (c) 2010 Volvox Development Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
// Author: Konstantin Lepa <konstantin.lepa@gmail.com>

#ifndef XMLPARSER_XML_ELEMENT_HPP_
#define XMLPARSER_XML_ELEMENT_HPP_

/**
	@file xml-element.hpp
	XML element class
 */

#include "xml-value.hpp"

#include <string>
#include <map>
#include <vector>

namespace simplexml
{

/**
	@addtogroup xmlgroup
	@{
 */

/**
	Config tree element.
 */
class XMLElement
{

public:
	typedef std::map<std::string, XMLValue> AttrMap;
	typedef std::vector<XMLElement *> ChildList;
	typedef ChildList::size_type ChildListSize;

public:
	/**
		Constructor
		@param parent - [in] parent element
		@param name - [in] element name
		@param line - [in] element line number in input data
		@param column - [in] element column number in input data
	 */
	XMLElement(XMLElement *parent, const std::string &name,
	           UINT_32 line, UINT_32 column);

	/**
		Get parent element
		@return parent element
	 */
	XMLElement * Parent() const;

	/**
		Get node name
		@return node name
	 */
	const std::string &Name() const;

	/**
		Check if node has specified attribute
		@param attName - [in] attribute name
		@return true if attribute exists, false otherwise
	 */
	bool HasAttr(const std::string &attName) const;

	/**
		Check if node has specified attribute as default
		@param attName - [in] attribute name
		@return true if attribute exists, false otherwise
	 */
	bool HasDefaultAttr(const std::string &attName) const;

	/**
		Get attribute value
		@param attName - [in] attribute name
		@return attribute value
		@throws XMLException
	 */
	const XMLValue & GetAttr(const std::string &attName) const;

	/**
		Set attribute value
		@param attName - [in] attribute name
		@param value - [in] attribute value
		@param hasDefault - [in] flag of default value
	 */
	void SetAttr(const std::string &attName, const std::string &value,
	             bool hasDefault = false);

	/**
		Access attributes
		@return attributes map
	 */
	const AttrMap &Attrs() const;

	/**
		Check if node has children
		@param name - [in] child name (empty value means all children)
		@return 0 - node has children, amount of children with
		            specified name otherwise
	 */
	UINT_32 HasChildren(const std::string &name = "") const;

	/**
		Get node children
		@return node children list
	 */
	const ChildList &Children() const;

	/**
		Get node child
		@param name - [in] child name
		@param num - [in] number of child
		@return node children list

		E.x.:

		<code>

		UINT_32 count = root->HasChildren("child");

		for (UINT_32 i = 0; i < count; ++i)
		{
			const simplexml::XMLElement *child =
				root->Child("child", i);

			std::cout << "Child value: ";
			std::cout << child->Child("param1")->GetAttr("value");
			std::cout << std::endl;
		}

		</code>
	 */
	XMLElement * Child(const std::string &name, UINT_32 num) const;

	/**
		Get child
		@param name - [in] child name
		@return node children list
	 */
	XMLElement * Child(const std::string &name) const;

	/**
		Add new child
		@param newChild - [in] child to add
	 */
	void AddChild(XMLElement *newChild);

	/**
		Get line in file
	 */
	UINT_32 LineInFile() const;

	/**
		Get column in file
	 */
	UINT_32 ColumnInFile() const;

	~XMLElement() throw();

private:
	XMLElement       *parent_;	///< Element parent node, NULL for root
	std::string       name_;	///< Element name
	UINT_32           line_;	///< Line number from source data
	UINT_32           column_;	///< Column number from source data
	AttrMap           attrs_;	///< Element attributes
	ChildList         children_;	///< Children list

	std::vector<std::string> defaultAttrs_;
}; // class XMLElement

/**@} */

} // namespace simplexml

#endif // XMLPARSER_XML_ELEMENT_HPP_

