// Copyright (c) 2010 Volvox Development Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
// Author: Konstantin Lepa <konstantin.lepa@gmail.com>

#ifndef XMLPARSER_XML_EXCEPTION_HPP_
#define XMLPARSER_XML_EXCEPTION_HPP_

/**
	@file xml-exception.hpp
	XML parse and validate exception
 */

#include "types.hpp"

#include <exception>
#include <string>
#include <stdio.h>
#include <limits>

namespace simplexml
{

/**
	@addtogroup xmlgroup
	@{
 */

/**
	XML exception
 */
class XMLException : public std::exception
{
public:
	XMLException() { ;; }

	/**
		@param msg - [in] error message
	 */
	XMLException(const std::string &msg)
	{
		error_.append("XML parse error: ");
		error_.append(msg);
	}

	/**
		Constructor, that added info of xml file (line and column).
		@param msg - [in] error message
		@param line - [in] line with error
		@param col - [in] column with error
	 */
	XMLException(const std::string &msg, UINT_32 line, UINT_32 col)
	{
		UINT_32 strSize = std::numeric_limits<UINT_32>::digits + 1;

		CHAR_P lineStr = new CHAR_8[strSize];
		CHAR_P colStr = new CHAR_8[strSize];

		snprintf(lineStr, strSize, "%d", line);
		snprintf(colStr, strSize, "%d", col);

		error_.append("XML parse error");
		if (line != 0 &&col != 0)
		{
			error_.append(" [line: ");
			error_.append(lineStr);
			error_.append(", col: ");
			error_.append(colStr);
			error_.append("]");
		}
		error_.append(": ");
		error_.append(msg);

		delete [] lineStr;
		delete [] colStr;
	}

	virtual ~XMLException() throw() { ;; }

	/**
		Get error description
		@return error text
	 */
	virtual const char *what() const throw() { return error_.c_str(); }

protected:
	std::string error_;	///< Error description

}; // class XMLException


/**
	XML exception for include file.
 */
class XMLIncludeException : public simplexml::XMLException
{
public:
	/**
		@param msg - [in] error message
		@param file - [in] file with error
		@param line - [in] line with error
	 */
	XMLIncludeException(const std::string &msg,
	             const std::string &file,
	             const UINT_32 line)
	{

		UINT_32 lineStrSize = std::numeric_limits<UINT_32>::digits + 1;

		CHAR_P lineStr = new CHAR_8[lineStrSize];

		snprintf(lineStr, lineStrSize, "%d", line);

		error_.append("XML parse include error: ");
		error_.append("included file ");
		error_.append(file);
		error_.append(": line ");
		error_.append(lineStr);
		error_.append(": ");
		error_.append(msg);

		delete [] lineStr;
	}

	virtual ~XMLIncludeException() throw() { ;; }

}; // class XMLIncludedException


/**
	XML exception for validator.
 */
class XMLValidatorException : public simplexml::XMLException
{
public:
	/**
		Constructor. Exception with info about invalid enitity.
		@param msg - [in] error message
		@param entity - [in] entity, that call error
	 */
	XMLValidatorException(const std::string &msg,
	                      const std::string &entity)
	{
		error_.append("XML validate error: ");
		error_.append(msg);
		error_.append(": ");
		error_.append(entity);
	}

	/**
		@param msg - [in] error message
		@param entity - [in] entity, that call error
		@param line - [in] line in the file
		@param col - [in] column in the file
	 */
	XMLValidatorException(const std::string &msg,
	                      const std::string &entity,
	                      UINT_32 line, UINT_32 col)
	{
		UINT_32 strSize = std::numeric_limits<UINT_32>::digits + 1;

		CHAR_P lineStr = new CHAR_8[strSize];
		CHAR_P colStr = new CHAR_8[strSize];

		snprintf(lineStr, strSize, "%d", line);
		snprintf(colStr, strSize, "%d", col);

		error_.append("XML validate error");
		if (line != 0 &&col != 0)
		{
			error_.append(" [tag - line: ");
			error_.append(lineStr);
			error_.append(", col: ");
			error_.append(colStr);
			error_.append("]");
		}
		error_.append(": ");
		error_.append(msg);
		error_.append(": ");
		error_.append(entity);

		delete [] lineStr;
		delete [] colStr;
	}

	virtual ~XMLValidatorException() throw() { ;; }

}; // class XMLValidatorException

/** @} */

} // namespace simplexml

#endif // XMLPARSER_XML_EXCEPTION_HPP_

