// Copyright (c) 2010 Volvox Development Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
// Author: Konstantin Lepa <konstantin.lepa@gmail.com>

#ifndef XMLPARSER_XML_HANDLER_HPP_
#define XMLPARSER_XML_HANDLER_HPP_

/**
	@file xml-handler.hpp
	XML document handler class
*/

#include "xml-handler-base.hpp"

namespace simplexml
{

/**
	@addtogroup xmlgroup
	@{
 */

class XMLElement;
class XMLValidator;

/**
	Config loader.
 */
class XMLHandler : public XMLHandlerBase
{
public:
	/**
		Build xml handler on xml element as root element
	 */
	XMLHandler(XMLElement *rootElem);

	XMLHandler();
	~XMLHandler() throw();

	/**
		Get root element
		@return root element

		@todo IMPORTANT: Must return const XMLElement!
	 */
	XMLElement * Root() const { return root_; }

	/**
		Start of XML Element
		@param elem - [in] tag name
		@param level - [in] tag level
		@param attr - [in] tag attributes
		@param line - [in] line
		@param column - [in] position
		@return 0 - ok, -1 - error
	 */
	virtual INT_32 StartElement(const std::string &elem, UINT_32 level,
	                            const std::vector<XMLAttr> &attr,
	                            UINT_32 line, UINT_32 column);

	/**
		End of XML Element
		@param elem - [in] tag name
		@param level - [in] tag level
		@param line - [in] line
		@param column - [in] position
		@return 0 - ok, -1 - error
	 */
	virtual INT_32 EndElement(const std::string &elem, UINT_32 level,
	                          UINT_32 line, UINT_32 column);

	/**
		Characters data
		@param data - [in] character data
		@param level - [in] tag level
		@param line - [in] line
		@param column - [in] position
		@return 0 - ok, -1 - error
	 */
	virtual INT_32 Characters(const std::string &data, UINT_32 level,
	                          UINT_32 line, UINT_32 column);

	/**
		Parsing error
		@param reason - [in] error description
		@param line - [in] line
		@param column - [in] position
	 */
	virtual void ParseError(const std::string &reason,
	                        UINT_32 line, UINT_32 column);

	/**
		Validate xml tree
		@param validator - [in] validator
		@throws simplexml::XMLException
	 */
	virtual void Validate(const XMLValidator &validator) const;

private:
	XMLElement *currNode_;	///< for StartElement() and EndElement() calls
	XMLElement *root_;	///< root element

	bool copied_;
	XMLHandler(const XMLHandler &);

}; // class XMLHandler

/** @} */

} // namespace simplexml

#endif // XMLPARSER_XML_HANDLER_HPP_

